@@ -38,11 +38,12 @@ ZEND_DECLARE_MODULE_GLOBALS(geoip)
 static int le_geoip;
 
 /* {{{ */
-function_entry geoip_functions[] = {
+zend_function_entry geoip_functions[] = {
 	PHP_FE(geoip_database_info,   NULL)   
-	PHP_FE(geoip_country_code_by_name,   NULL)
-	PHP_FE(geoip_country_code3_by_name,   NULL)
-	PHP_FE(geoip_country_name_by_name,   NULL)
+#define GEOIPDEF(php_func, c_func, db_type) \
+	PHP_FE(php_func,	NULL)
+#include "geoip.def"
+#undef GEOIPDEF
 	PHP_FE(geoip_continent_code_by_name,   NULL)
 	PHP_FE(geoip_org_by_name,   NULL)
 	PHP_FE(geoip_record_by_name,   NULL)
@@ -237,7 +238,9 @@ PHP_FUNCTION(geoip_db_get_all_info)
 
 			add_assoc_bool(row, "available", GeoIP_db_avail(i));
 			add_assoc_string(row, "description", (char *)GeoIPDBDescription[i], 1);
-			add_assoc_string(row, "filename", GeoIPDBFileName[i], 1);
+			if (GeoIPDBFileName[i]) {
+				add_assoc_string(row, "filename", GeoIPDBFileName[i], 1);
+			}
 
 			add_index_zval(return_value, i, row);
 		}
@@ -281,94 +284,37 @@ PHP_FUNCTION(geoip_database_info)
 }
 /* }}} */
 
-/* {{{ proto string geoip_country_code_by_name( string hostname )
-   Return the Country Code found in the GeoIP Database */
-PHP_FUNCTION(geoip_country_code_by_name)
-{
-	GeoIP * gi;
-	char * hostname = NULL;
-	const char * country_code;
-	int arglen;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &hostname, &arglen) == FAILURE) {
-		return;
+/* {{{ */
+#define GEOIPDEF(php_func, c_func, db_type) \
+	PHP_FUNCTION(php_func) \
+	{ \
+		GeoIP * gi; \
+		char * hostname = NULL; \
+		const char * return_code; \
+		int arglen; \
+		\
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &hostname, &arglen) == FAILURE) { \
+			return; \
+		} \
+		\
+		if (GeoIP_db_avail(db_type)) { \
+			gi = GeoIP_open_type(db_type, GEOIP_STANDARD); \
+		} else { \
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Required database not available at %s.", GeoIPDBFileName[db_type]); \
+			return; \
+		} \
+		\
+		return_code = c_func(gi, hostname); \
+		GeoIP_delete(gi); \
+		if (return_code == NULL) { \
+			php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Host %s not found", hostname); \
+			RETURN_FALSE; \
+		} \
+		RETURN_STRING((char*)return_code, 1); \
+		\
 	}
-
-	if (GeoIP_db_avail(GEOIP_COUNTRY_EDITION)) {
-		gi = GeoIP_open_type(GEOIP_COUNTRY_EDITION, GEOIP_STANDARD);
-	} else {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Required database not available at %s.", GeoIPDBFileName[GEOIP_COUNTRY_EDITION]);
-		return;
-	}
-	
-	country_code = GeoIP_country_code_by_name(gi, hostname);
-	GeoIP_delete(gi);
-	if (country_code == NULL) {
-		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Host %s not found", hostname);
-		RETURN_FALSE;
-	}
-	RETURN_STRING((char*)country_code, 1);
-}
-/* }}} */
-
-/* {{{ proto string geoip_country_code_by_name( string hostname )
-   Return the Country Code found in the GeoIP Database */
-PHP_FUNCTION(geoip_country_code3_by_name)
-{
-	GeoIP * gi;
-	char * hostname = NULL;
-	const char * country_code;
-	int arglen;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &hostname, &arglen) == FAILURE) {
-		return;
-	}
-
-	if (GeoIP_db_avail(GEOIP_COUNTRY_EDITION)) {
-		gi = GeoIP_open_type(GEOIP_COUNTRY_EDITION, GEOIP_STANDARD);
-	} else {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Required database not available at %s.", GeoIPDBFileName[GEOIP_COUNTRY_EDITION]);
-		return;
-	}
-	
-	country_code = GeoIP_country_code3_by_name(gi, hostname);
-	GeoIP_delete(gi);
-	if (country_code == NULL) {
-		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Host %s not found", hostname);
-		RETURN_FALSE;
-	}
-	RETURN_STRING((char*)country_code, 1);
-}
-/* }}} */
-
-/* {{{ proto string geoip_country_name_by_name( string hostname )
-   Returns the Country name found in the GeoIP Database */
-PHP_FUNCTION(geoip_country_name_by_name)
-{
-	GeoIP * gi;
-	char * hostname = NULL;
-	const char * country_name;
-	int arglen;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &hostname, &arglen) == FAILURE) {
-		return;
-	}
-
-	if (GeoIP_db_avail(GEOIP_COUNTRY_EDITION)) {
-		gi = GeoIP_open_type(GEOIP_COUNTRY_EDITION, GEOIP_STANDARD);
-	}   else {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Required database not available at %s.", GeoIPDBFileName[GEOIP_COUNTRY_EDITION]);
-		return;
-	}
-
-	country_name = GeoIP_country_name_by_name(gi, hostname);
-	GeoIP_delete(gi);
-	if (country_name == NULL) {
-		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Host %s not found", hostname);
-		RETURN_FALSE;
-	}
-	RETURN_STRING((char*)country_name, 1);
-}
+#include "geoip.def"
+#undef GEOIPDEF
 /* }}} */
 
 /* {{{ proto string geoip_continent_code_by_name( string hostname )
@@ -588,7 +534,6 @@ PHP_FUNCTION(geoip_isp_by_name)
    Returns the region name for some country code and region code combo */
 PHP_FUNCTION(geoip_region_name_by_code)
 {
-	GeoIP * gi;
 	char * country_code = NULL;
 	char * region_code = NULL;
 	const char * region_name;
@@ -617,7 +562,6 @@ PHP_FUNCTION(geoip_region_name_by_code)
    Returns the time zone for some country code and region code combo */
 PHP_FUNCTION(geoip_time_zone_by_country_and_region)
 {
-	GeoIP * gi;
 	char * country = NULL;
 	char * region = NULL;
 	const char * timezone;
